import requests
from datetime import datetime, timedelta
import time

# Parameter definitions
TOKEN = 'Your PropellerAds API token'  # API token for authentication
IMPRESSION_THRESHOLD = 200  # Minimum number of impressions for consideration
PERFORMANCE_DAYS = 0  # Number of days to fetch performance data
BLACKLIST_DAYS = 7  # Number of days to blacklist campaigns
PROFIT_THRESHOLD = -3.5  # Default profit threshold for stopping campaigns
ROI_THRESHOLD = -45  # Default ROI threshold for stopping campaigns
INTERVAL = 60  # Interval between each iteration, in seconds
TIMEOUT = 15  # Timeout for API requests, in seconds

# Function to fetch active campaigns from the API
def get_active_campaign_ids():
    campaign_api_url = 'https://ssp-api.propellerads.com/v5/adv/campaigns?status%5B%5D=6'
    headers = {
        'accept': 'application/json',  # Accept JSON responses
        'Authorization': f'Bearer {TOKEN}'  # Include the API token for authentication
    }

    try:
        # Make a GET request to fetch active campaigns
        response = requests.get(campaign_api_url, headers=headers, timeout=TIMEOUT)
        response.raise_for_status()
        data = response.json()  # Parse the JSON response
    except requests.exceptions.Timeout:
        print(f"Request to {campaign_api_url} timed out after {TIMEOUT} seconds.")
        return []
    except requests.exceptions.RequestException as e:
        print(f"Error fetching active campaigns: {e}")
        return []

    # Extract and print campaign IDs and names
    campaigns = data.get('result', [])
    campaign_ids = []

    for campaign in campaigns:
        campaign_id = campaign.get('id')
        campaign_name = campaign.get('name')
        print(f'Campaign ID: {campaign_id}, Name: {campaign_name}')
        campaign_ids.append(campaign_id)

    print('All Active Campaign IDs:', campaign_ids)
    return campaign_ids

# Function to extract custom thresholds from campaign names
def extract_custom_threshold(campaign_name, key):
    key_part = f'{key}='
    if key_part in campaign_name:
        try:
            # Split the campaign name to extract the value for the specified key
            value = campaign_name.split(key_part)[1].split(']')[0].strip()
            return float(value)
        except (ValueError, IndexError):
            print(f'Invalid {key} value in campaign name: {campaign_name}')
    return None

# Function to fetch and analyze performance data for campaigns
def fetch_campaign_performance(campaign_ids):
    performance_api_url = 'https://ssp-api.propellerads.com/v5/adv/statistics'
    headers = {
        'accept': 'application/json',  # Accept JSON responses
        'Authorization': f'Bearer {TOKEN}',  # Include the API token for authentication
        'Content-Type': 'application/json'  # Specify the content type for the request
    }

    # Define the date range for fetching performance data
    today = datetime.now()
    seven_days_ago = today - timedelta(days=PERFORMANCE_DAYS)
    day_to = today.strftime('%Y-%m-%d 23:59:59')
    day_from = seven_days_ago.strftime('%Y-%m-%d 00:00:00')

    data = {
        "group_by": ["campaign_id"],
        "day_from": day_from,
        "day_to": day_to,
        "tz": "+0800",
        "campaign_id": campaign_ids,
        "geo": [],
        "dept": [],
        "rate_model_id": [],
        "impressions": {},
        "order_dest": "desc"
    }

    try:
        # Make a POST request to fetch performance data
        response = requests.post(performance_api_url, headers=headers, json=data, timeout=TIMEOUT)
        response.raise_for_status()
        performance_data = response.json()  # Parse the JSON response
    except requests.exceptions.Timeout:
        print(f"Request to {performance_api_url} timed out after {TIMEOUT} seconds.")
        return
    except requests.exceptions.RequestException as e:
        print(f"Error fetching campaign performance: {e}")
        return

    campaigns_to_stop = []

    # Analyze performance data for each campaign
    for campaign in performance_data:
        profit = campaign.get('profit', 0)
        roi = campaign.get('roi', 0)
        campaign_id = campaign.get('campaign_id')
        campaign_name = campaign.get('campaign_name')
        campaign_impression = campaign.get('impressions')
        print(
            f"Profit: {profit:>5} | ROI: {roi:>4} | Campaign ID: {campaign_id:>8} | Impressions: {campaign_impression:>8} | Campaign Name: {campaign_name}")

        # Default thresholds
        custom_profit_threshold = PROFIT_THRESHOLD
        custom_roi_threshold = ROI_THRESHOLD

        # Check for custom thresholds in the campaign name
        custom_profit_threshold = extract_custom_threshold(campaign_name, 'PROFIT') or PROFIT_THRESHOLD
        custom_roi_threshold = extract_custom_threshold(campaign_name, 'ROI') or ROI_THRESHOLD

        # Log custom thresholds if specified
        if custom_profit_threshold is not None or custom_roi_threshold is not None:
            print(
                f'Campaign Name: {campaign_name} | Custom PROFIT_THRESHOLD: {custom_profit_threshold} | Custom ROI_THRESHOLD: {custom_roi_threshold}')

        # Add campaigns to stop list if they fall below thresholds
        if profit < custom_profit_threshold and roi < custom_roi_threshold:
            print(f'Campaign Name: {campaign_name} is abnormal!!!!!!!')
            campaigns_to_stop.append(campaign_id)

    # Stop campaigns that are not performing well
    if campaigns_to_stop:
        stop_campaigns(campaigns_to_stop)
    else:
        print('No campaigns to stop.')

# Function to stop campaigns
def stop_campaigns(campaign_ids_in):
    stop_api_url = 'https://ssp-api.propellerads.com/v5/adv/campaigns/stop'
    headers = {
        'accept': 'application/json',  # Accept JSON responses
        'Authorization': f'Bearer {TOKEN}',  # Include the API token for authentication
        'Content-Type': 'application/json'  # Specify the content type for the request
    }

    payload = {
        "campaign_ids": campaign_ids_in
    }

    try:
        # Make a PUT request to stop campaigns
        response = requests.put(stop_api_url, headers=headers, json=payload, timeout=TIMEOUT)
        response.raise_for_status()
        print(f'Stop Campaigns Response:', response.json())
    except requests.exceptions.Timeout:
        print(f"Request to {stop_api_url} timed out after {TIMEOUT} seconds.")
    except requests.exceptions.RequestException as e:
        print(f"Error stopping campaigns: {e}")

# Function to print the current time
def print_current_time():
    now = datetime.now()
    current_time = now.strftime('%Y-%m-%d %H:%M:%S')
    print(f'Current Time: {current_time}')

# Main loop
while True:
    print('Fetching active campaigns: ')
    active_campaign_ids = get_active_campaign_ids()
    if active_campaign_ids:
        print('Analyzing and stopping underperforming campaigns: ')
        fetch_campaign_performance(active_campaign_ids)
    else:
        print('No active campaigns found.')

    print_current_time()
    print(f'Waiting {INTERVAL} seconds before next iteration...')
    time.sleep(INTERVAL)
